@@ -41,6 +41,10 @@ use Rcmd;
 use G::Messenger;
 use G::Inspire;
 
+# import shell functions
+use G::Shell::EUtils;
+use G::Shell::Help;
+
 # import core modules
 use G::DynamicLoader;
 
@@ -115,6 +119,8 @@ push(@EXPORT, @G::Tools::GMap::EXPORT);
 push(@EXPORT, @G::Tools::WebServices::EXPORT);
 push(@EXPORT, @G::Tools::DotE::EXPORT);
 
+push(@EXPORT, @G::Shell::EUtils::EXPORT);
+push(@EXPORT, @G::Shell::Help::EXPORT);
 push(@EXPORT, @G::DB::SDB::EXPORT);
 push(@EXPORT, @G::DB::BDB::EXPORT);
 push(@EXPORT, @SubOpt::EXPORT);
@@ -134,6 +140,7 @@ sub load{
 sub opt_list{
     my $sub = shift;
 
+    SubOpt::opt_default();
     SubOpt::set_opt_list(1);
     eval("&{$sub}");
     SubOpt::set_opt_list(0);
@@ -147,7 +154,7 @@ sub method_list{
 
     for my $name (qw/
 		  p puts say readFile writeFile 
-		  opt_as_gb opt_default opt_get opt_list opt_val method_list
+		  opt_as_gb opt_default opt_get opt_list opt_val
 		  msg_ask_interface msg_error msg_send msg_gimv msg_interface msg_percent msg_progress msg_set_gimv msg_system_console msg_term_console
 		  sdb_exists sdb_load sdb_save _sdb_path _set_sdb_path
 		  db_dbi db_exists db_load db_path db_save db_set_path
@@ -156,8 +163,16 @@ sub method_list{
 	$system{$name} ++;
     }
 
-    if($opt){
+    if($opt eq 'system'){
 	return (sort keys %system);
+    }elsif($opt eq 'gb'){
+	return sort qw/output seq seq_info find getseq get_gbkseq get_cdsseq
+	    get_geneseq feature cds tRNA rRNA intergenic gene disable_pseudogenes next_feature
+	    next_cds previous_feature previous_cds startcodon stopcodon before_startcodon
+	    after_startcodon before_stopcodon after_stopcodon around_startcodon around_stopcodon
+	    get_exon get_intron pos2feature pos2gene gene2id next_locus clone del_key
+	    reverse_strand relocate_origin
+	    /;
     }else{
 	my @return;
 
@@ -194,7 +209,7 @@ G - G-language Genome Analysis Environment core interface module
    
  $gb->seq_info();                # Prints the basic sequence information.
 
- $find_ori_ter($gb);             # Give $gb as the first argument to 
+ find_ori_ter($gb);              # Give $gb as the first argument to 
                                  # most of the analysis functions
 
 
@@ -215,6 +230,19 @@ G - G-language Genome Analysis Environment core interface module
 
  HEADER  
          $gb->{HEADER}  
+         $gb->{DEFINITION}
+         $gb->{ACCESSION}
+         $gb->{SOURCE}
+         $gb->{ORGANISM}
+
+         $gb->{TAXONOMY}->{all}          -same as $gb->{TAXONOMY}->{1}
+         $gb->{TAXONOMY}->{domain}       -same as $gb->{TAXONOMY}->{2}
+         $gb->{TAXONOMY}->{phylum}       -same as $gb->{TAXONOMY}->{3}
+         $gb->{TAXONOMY}->{class}        -same as $gb->{TAXONOMY}->{4}
+         $gb->{TAXONOMY}->{order}}       -same as $gb->{TAXONOMY}->{5}
+         $gb->{TAXONOMY}->{family}       -same as $gb->{TAXONOMY}->{6}
+         $gb->{TAXONOMY}->{genus}
+         $gb->{TAXONOMY}->{species}
 
  COMMENT  
          $gb->{COMMENT}  
@@ -564,6 +592,15 @@ G - G-language Genome Analysis Environment core interface module
    Description:
          Returns the array of all feature IDs of rRNAs.
 
+   Options:
+         Second optional argument can specify the rRNA type 
+         (5S, 16S, 23S, 5.8S, 18S, 28S). When used in scalar context,
+         only the longest match is returned.
+
+         ex: 
+            @SSU = $gb->rRNA('16S');
+            $longestSSU = $gb->rRNA('16S');
+
 =cut
 
 =head2 $gb->intergenic()
