@@ -42,8 +42,9 @@ require G::IO::EmblI;
 
 use File::Basename;
 use File::ShareDir ':ALL';
+use autouse 'File::Temp'=>qw(tempfile);
 
-our $VERSION = '1.8.8';
+our $VERSION = '1.8.9';
 
 #::::::::::::::::::::::::::::::
 #          Perldoc
@@ -51,12 +52,12 @@ our $VERSION = '1.8.8';
 
 =head1 NAME
 
-G::IO - G-language v.2 core class
+G::IO - G-language System v.2 core class
 
 =head1 SYNOPSIS
 
  use G::IO;
- @ISA = (G::IO);
+ use base qw(G::IO);
    
 =head1 DESCRIPTION
 
@@ -108,18 +109,18 @@ sub new {
     
     bless $this;
 
-    my $no_msg = 0;
-    my $without_annotation = 0;
-    my $multiple_locus = 0;
-    my $long_sequence = 0;
-    my $bioperl = 0;
-    my $netgbk = 0;
+    my $no_msg                 = 0;
+    my $without_annotation     = 0;
+    my $multiple_locus         = 0;
+    my $long_sequence          = 0;
+    my $bioperl                = 0;
+    my $netgbk                 = 0;
     my $longest_ORF_annotation = 0;
-    my $glimmer_annotation = 0;
-    my $no_cache = 0;
-    my $force_cache = 0;
-    my $format = '';
-    my $locus_msg = '';
+    my $glimmer_annotation     = 0;
+    my $no_cache               = 0;
+    my $force_cache            = 0;
+    my $format                 = '';
+    my $locus_msg              = '';
 
     return $this if ($filename eq 'blessed');
 
@@ -169,7 +170,7 @@ sub new {
 
     $this->loaded_msg() unless ($no_msg);
 
-    $format = interpret_format($filename) unless(length $format);
+    $format = G::IO::Handler::_interpret_format($filename) unless(length $format);
 
     if($format eq 'RefSeq'){
 	$format = 'usa';
@@ -219,7 +220,7 @@ sub new {
 	    $this = new G::IO::FastaI;
 	}elsif ($format ne 'genbank'){
 
-	    $outfile = _generate_tmpfilename();
+	    (undef, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
 	    my $in = read_sequence($filename);
 	    my $out = write_sequence(">$outfile", "genbank", $in);
 
@@ -234,9 +235,6 @@ sub new {
 	    my $tmp = $this->clone();
 	    undef(%$this);
 
-#	    $this = new G::IO("blessed");
-#	    my $tmp = new G::IO($filename, "no msg", $format, "no cache");
-
 	    multi_locus($this, $tmp);
 	    
 	    $this->set_gene_aliases();
@@ -324,87 +322,13 @@ sub loaded_msg {
     &msg_error($print);
 }
 
-sub interpret_format {
-    my $filename = shift;
-    my $ref = ref $filename;
-    if ($ref =~ /Bio::Seq/){
-	return 'bioperl';
-    }elsif ($filename =~ /^(.*?):(.*)/){
-	unless(lc($1) =~ /(swiss|genbank|genpept|embl|refseq)/){
-	    die("Unsupported database name for USA. Supported databases are\n" .
-		"swiss, genbank, genpept, embl, or refseq\n");
-	}
-	return "usa";
-    }elsif (lc($filename) =~ /\.(gb|gbk|gbank|genbank)$/i){
-	return 'genbank';
-    }elsif (lc($filename) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa)/i){
-	return 'fasta';
-    }elsif ($filename =~ /^NC_\d+$/i){
-	return 'RefSeq';
-    }elsif ($filename =~ /^NP_\d+$/i){
-	return 'net GenPept';
-    }elsif ($filename =~ /^(?:[A-Z]|[A-Z][A-Z]).\d+$/i){
-	return 'net GenBank';
-    }else{
-	require Bio::SeqIO;
-	my $format = Bio::SeqIO->_guess_format($filename);
-
-	if (length $format){
-	    return $format;
-	}else{
-	    warn("Unknown file format. Interpreting $filename as GenBank...\n");
-	    return 'genbank';
-	}
-    }
-}
-
-sub output {
-    my $gb = shift;
-    my $file = shift;
-    my $option = shift;
-
-    $option = interpret_format($file) unless(length $option);
-
-    if (lc($option) eq 'genbank'){
-	$gb->make_gb($file);
-    }elsif(length $option){
-	my $in;
-
-	if(length $gb->{BIOPERL}){
-	    $in = $gb->{BIOPERL};
-	}else{
-	    my $outfile = _generate_tmpfilename();
-	    $gb->make_gb($outfile);
-	    $in = read_sequence($outfile);
-	    unlink($outfile);
-	}
-
-	write_sequence(">$file", $option, $in);
-    }else{
-	&msg_error("G::output - Unknown format to output.");
-    }
-}
-
-sub bioperl {
-    require Bio::SeqIO;
-    my $this = shift;
-    my $outfile = _generate_tmpfilename();
-    $this->output($outfile);
-    
-    my $in = Bio::SeqIO->new(-format => "genbank", -file =>$outfile);
-    my $bp = $in->next_seq();
-    unlink($outfile);
-    return $bp;
-}
 
 sub _bp2gb {
     my $bpobj = shift;
 
-    my $tmpname = "G-INTERNAL-" . $ENV{USER} . time() . rand();
-    write_sequence(">/tmp/$tmpname.gbk", "genbank", $bpobj);
-
-    my $this = new G::IO("/tmp/$tmpname.gbk", "no msg");
-    unlink("/tmp/$tmpname.gbk");
+    my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
+    write_sequence('>' . $outfile, "genbank", $bpobj);
+    my $this = new G::IO($outfile, "no msg");
 
     return $this;
 }
@@ -468,17 +392,12 @@ sub multi_locus{
     $this->{"COMMENT"} = $this->{"COMMENT1"};
 }
 
-
-sub _generate_tmpfilename{
-    return '/tmp/G-INTERNAL-' . $ENV{USER} . time() . rand() .'.gbk';
-}
-
 sub DESTROY {
     my $self = shift;
 
     undef %{$self};
-    unlink($outfile);
     $self->close_gb();
+    unlink($outfile);
 }
 
 
