@@ -36,6 +36,8 @@ use G::Messenger;
 #     Let the code begin...
 #::::::::::::::::::::::::::::::
 
+=head 
+
 sub output {
     my $gb = shift;
     my $file = shift;
@@ -43,6 +45,8 @@ sub output {
     make_gb($gb, $file);
 }
 
+=cut 
+
 sub make_gb {
     my $gb = shift;
     my $file = shift;
@@ -61,11 +65,11 @@ sub make_gb {
         $type = "linear";
     }
 
-    my @locus = ("LOCUS", $gb->{LOCUS}->{id}, "$gb->{LOCUS}->{length}bp", 
+    my @locus = ("LOCUS", $gb->{LOCUS}->{id}, "$gb->{LOCUS}->{length} bp", 
 		 $gb->{LOCUS}->{nucleotide}, $type, $gb->{LOCUS}->{type}, 
 		 $gb->{LOCUS}->{date});
 
-    printf OUT "%-11.11s %-10.10s %-12.12s %-5.5s %-9.9s %-9.9s%-10.10s\n",@locus;
+    printf OUT "%-11.11s %-15.15s %15.15s    %-7.7s %-8.8s %-3.3s %11.11s\n",@locus;
     printf OUT "%s%sFEATURES %11s Location/Qualifiers\n", $gb->{HEADER}, $gb->{COMMENT};
 
     foreach my $feat ($gb->feature()){
@@ -73,7 +77,7 @@ sub make_gb {
             if($gb->{$feat}->{join}){
 
 		my $tmpJoinLine = $gb->{$feat}->{join};
-
+		$z = 0;
 		my @tmpjoin = ();
 		foreach my $joinsegment (split(/,/, $tmpJoinLine)){
 		    if($joinsegment =~ /c/){
@@ -84,50 +88,81 @@ sub make_gb {
 		    }
 		}
 		
-		$tmpJoinLine = join(',', @tmpjoin);
-
-		my $join = "join"."(". $tmpJoinLine .")";
-		my $position = rindex($join,',',58);
-		for($z = 0; $z <= length($join); $z += 58){
-                    my $join_cut = substr($join,$z,58);
-
-                    if($z == 0){
-			printf OUT "%-4.4s %-15.15s %-58.58s\n","$dummy","$gb->{$feat}->{type}","$join_cut";
-                    }else{
-                        printf OUT "%-20.20s %-58.58s\n","$dummy","$join_cut";
-                    }
+		my $join = 'join(';
+		for $tmpJoinLine (@tmpjoin){
+		    if(length($join) + length($tmpJoinLine) <=58){
+			$join .= $tmpJoinLine . ',';
+		    }else{
+			if($z == 0){
+			    printf OUT "%-4.4s %-15.15s %s\n", $dummy, $gb->{$feat}->{type}, $join;
+			}else{
+			    printf OUT "%-20.20s %s\n", $dummy, $join;
+			}
+			$z ++;
+			$join = $tmpJoinLine . ',';
+		    }
 		}
+
+		substr($join, -1, 1) = ')';
+		if($z == 0){
+		    printf OUT "%-4.4s %-15.15s %s\n", $dummy, $gb->{$feat}->{type}, $join;
+		}else{
+		    printf OUT "%-20.20s %s\n", $dummy, $join;
+		}
+			
             }else{
                 my @partial = split(/ /,$gb->{$feat}->{"partial"});
                 if($partial[1] == 1){
-                    printf OUT "%-4.4s %-15.15s %-58.58s\n",
+                    printf OUT "%-4.4s %-15.15s %s\n",
 			    "$dummy","$gb->{$feat}->{type}",
 			    "$gb->{$feat}->{start}..>$gb->{$feat}->{end}";
                 }elsif($partial[0] == 1){
-                    printf OUT "%-4.4s %-15.15s %-58.58s\n",
+                    printf OUT "%-4.4s %-15.15s %s\n",
 			    "$dummy","$gb->{$feat}->{type}",
 			    "<$gb->{$feat}->{start}..$gb->{$feat}->{end}";
                 }else{
-                    printf OUT "%-4.4s %-15.15s %-58.58s\n","$dummy",
+                    printf OUT "%-4.4s %-15.15s %s\n","$dummy",
 			    "$gb->{$feat}->{type}",
 			    "$gb->{$feat}->{start}..$gb->{$feat}->{end}";
 		}
             }
 	}elsif($gb->{$feat}->{"direction"} ne "direct"){
             if($gb->{$feat}->{join}){
-		my $join = $gb->{$feat}->{"direction"}."("."join"."(".$gb->{$feat}->{join}.")".")";
-		
-                for($z = 0; $z < length($join); $z += 58){
-                    my $join_cut = substr($join,$z,58);
-                    if($z == 0){
-                        printf OUT "%-4.4s %-15.15s %-58.58s\n",
-			       "$dummy","$gb->{$feat}->{type}","$join_cut";
-                    }else{
-                        printf OUT "%-20.20s %-58.58s\n","$dummy","$join_cut";
+		my $tmpJoinLine = $gb->{$feat}->{join};
+		$z = 0;
+		my @tmpjoin = ();
+		foreach my $joinsegment (split(/,/, $tmpJoinLine)){
+		    if($joinsegment =~ /c/){
+			$joinsegment =~ s/c//g;
+			push(@tmpjoin, 'complement(' . $joinsegment . ')');
+		    }else{
+			push(@tmpjoin, $joinsegment);
 		    }
 		}
-            }else{
-                printf OUT "%-4.4s %-15.15s %-58.58s\n",
+
+		my $join = 'complement(join(';
+		for $tmpJoinLine (@tmpjoin){
+		    if(length($join) + length($tmpJoinLine) <=58){
+			$join .= $tmpJoinLine . ',';
+		    }else{
+			if($z == 0){
+			    printf OUT "%-4.4s %-15.15s %s\n", $dummy, $gb->{$feat}->{type}, $join;
+			}else{
+			    printf OUT "%-20.20s %s\n", $dummy, $join;
+			}
+			$z ++;
+			$join = $tmpJoinLine . ',';
+		    }
+		}
+		substr($join, -1, 1) = '))';
+		if($z == 0){
+		    printf OUT "%-4.4s %-15.15s %s\n", $dummy, $gb->{$feat}->{type}, $join;
+		}else{
+		    printf OUT "%-20.20s %s\n", $dummy, $join;
+		}
+
+	    }else{
+                printf OUT "%-4.4s %-15.15s %s\n",
 			"$dummy","$gb->{$feat}->{type}",
 			"$gb->{$feat}->{direction}($gb->{$feat}->{start}..$gb->{$feat}->{end})";
             }
@@ -141,7 +176,29 @@ sub make_gb {
 	    if($gb->{$feat}->{$key} =~ /\t/){
 		foreach my $multiplekey (split(/\t/, $gb->{$feat}->{$key})){
 		    my $str = "/" . $key . "=" . "\"" . $multiplekey . "\"";
-		    printf OUT "%-20.20s %-58.58s\n","$dummy",$str;
+
+		    $lng = length($str);
+		    if($lng >= 58 ){
+			if($str =~ /\s/){
+			    my $tmpline = "";
+			    foreach my $word (split(/ /, $str)){
+				if(length($tmpline) + length($word) < 58){
+				    $tmpline .= $word . ' ';
+				}else{
+				    printf OUT "%-20.20s %s\n",$dummy,$tmpline;
+				    $tmpline = $word . ' ';
+				}
+			    }
+			    printf OUT "%-20.20s %s\n",$dummy,$tmpline;
+			}else{
+			    for($i = 0;$i < $lng;$i += 58){
+				my $pr = substr($str,$i,58);
+				printf OUT "%-20.20s %s\n",$dummy,$pr;
+			    }
+			}
+		    }else{
+			printf OUT "%-20.20s %s\n","$dummy",$str;
+		    }
 		}
 	    }else{
 		my $str = "/".$key."="."\"".$gb->{$feat}->{$key}."\"";
@@ -153,25 +210,25 @@ sub make_gb {
 			    if(length($tmpline) + length($word) < 58){
 				$tmpline .= $word . ' ';
 			    }else{
-				printf OUT "%-20.20s %-58.58s\n",$dummy,$tmpline;
+				printf OUT "%-20.20s %s\n",$dummy,$tmpline;
 				$tmpline = $word . ' ';
 			    }
 			}
-			printf OUT "%-20.20s %-58.58s\n",$dummy,$tmpline;
+			printf OUT "%-20.20s %s\n",$dummy,$tmpline;
 		    }else{
 			for($i = 0;$i < $lng;$i += 58){
 			    my $pr = substr($str,$i,58);
-			    printf OUT "%-20.20s %-58.58s\n",$dummy,$pr;
+			    printf OUT "%-20.20s %s\n",$dummy,$pr;
 			}
 		    }
 		}else{
-		    printf OUT "%-20.20s %-58.58s\n","$dummy",$str;
+		    printf OUT "%-20.20s %s\n","$dummy",$str;
 		}
 	    }
 	}
     }
 
-    print OUT "BASE COUNT $gb->{BASE_COUNT}\n";
+#    print OUT "BASE COUNT $gb->{BASE_COUNT}\n";
     print OUT "ORIGIN\n";
     for($p = 0;$p<=length($gb->{SEQ});$p += 60){
 	my $seq_prt = "";
