@@ -34,6 +34,9 @@ use G::Messenger;
 use G::Seq::Primitive;
 use G::DB::SDB;
 
+use autouse 'File::Temp'=>qw(tempfile);
+use autouse 'Bio::Perl'=>qw(read_sequence write_sequence get_sequence);
+
 #::::::::::::::::::::::::::::::
 #    Variables
 #::::::::::::::::::::::::::::::
@@ -53,7 +56,7 @@ my $intergenic = 0;
 =head1 SYNOPSIS
 
  require G::IO::Handler;
- @ISA = (G::IO::Handler);
+ use base qw(G::IO::Handler);
    
 =head1 DESCRIPTION
 
@@ -110,6 +113,24 @@ sub clone {
     return $new;
 }
 
+sub disclose {
+    my $gb = shift;
+
+    my $return = '';
+    foreach my $level1 (keys %{$gb}){
+	next if ($level1 eq 'SEQ');
+	next unless($level1 =~ /LOCUS|FEATURE|HEADER/);
+	if(ref($gb->{$level1}) =~ /HASH/){
+	    foreach my $level2 (keys %{$gb->{$level1}}){
+		$return .= join("\t", $level1, $level2, $gb->{$level1}->{$level2}) . "\n";
+	    }
+	}else{
+	    $return .= join("\t", $level1, $gb->{$level1}) . "\n";
+	}
+    }
+
+    return $return;
+}
 
 
 sub find {
@@ -358,7 +379,7 @@ sub pos2feature {
 }
 
 sub gene2id {
-    return 'FEATURE' . $_[0]->{$_[1]}->{feature};
+    return length($_[0]->{$_[1]}->{feature}) ? 'FEATURE' . $_[0]->{$_[1]}->{feature} : '';
 }
 
 sub feature2id {
@@ -529,7 +550,40 @@ sub feature {
 }
 
 sub rRNA {
-    return feature($_[0], 'rRNA', $_[1]);
+    my $this = shift;
+    my $option = shift;
+
+    if($option =~ /\d+S/){
+	my %rRNA;
+
+	foreach ($this->feature()){
+	    if($this->{$_}->{product} =~ /([0-9\.]+S)/i){
+		push(@{$rRNA{uc($1)}}, $_);
+	    }elsif($this->{$_}->{gene} =~ /rrf/){
+		push(@{$rRNA{'5S'}}, $_);
+	    }elsif($this->{$_}->{gene} =~ /rrl/){
+		push(@{$rRNA{'23S'}}, $_);
+	    }elsif($this->{$_}->{gene} =~ /rrs/){
+		push(@{$rRNA{'16S'}}, $_);
+	    }elsif($this->{$_}->{gene} =~ /([0-9\.]+S)/i){
+		push(@{$rRNA{uc($1)}}, $_);
+	    }elsif($this->{$_}->{locus_tag} =~ /([0-9]+S)/i){
+		push(@{$rRNA{uc($1)}}, $_);
+	    }elsif($this->{$_}->{note} =~ /([0-9\.]+S)+?/){
+		push(@{$rRNA{uc($1)}}, $_);
+	    }elsif($this->{$_}->{product} =~ /(SSU|small subunit) ribosomal RNA/i){
+		push(@{$rRNA{'SSU'}}, $_);
+	    }elsif($this->{$_}->{product} =~ /(LSU|large subunit) ribosomal RNA/i){
+		push(@{$rRNA{'LSU'}}, $_);
+	    }
+	}
+
+	my @result = sort {length($this->get_geneseq($b))<=>length($this->get_geneseq($a))} @{$rRNA{$option}};
+
+	return wantarray ? @result : $result[0];
+    }else{
+	return feature($this, 'rRNA', $option);
+    }
 }
 
 sub tRNA {
@@ -829,6 +883,80 @@ sub reverse_strand{
 }
 
 
+sub _interpret_format {
+    my $filename = shift;
+    my $ref = ref $filename;
+
+    if ($ref =~ /Bio::Seq/){
+	return 'bioperl';
+    }elsif ($filename =~ /^(.*?):(.*)/){
+	unless(lc($1) =~ /(swiss|genbank|genpept|embl|refseq)/){
+	    die("Unsupported database name for USA. Supported databases are\n" .
+		"swiss, genbank, genpept, embl, or refseq\n");
+	}
+	return "usa";
+    }elsif (lc($filename) =~ /\.(gb|gbk|gbank|genbank)$/i){
+	return 'genbank';
+    }elsif (lc($filename) =~ /\.(fasta|fast|seq|fst|fa|fsa|nt|aa|ffn|fna)/i){
+	return 'fasta';
+    }elsif ($filename =~ /^NC_\d+$/i){
+	return 'RefSeq';
+    }elsif ($filename =~ /^NP_\d+$/i){
+	return 'net GenPept';
+    }elsif ($filename =~ /^(?:[A-Z]|[A-Z][A-Z]).\d+$/i){
+	return 'net GenBank';
+    }else{
+	require Bio::SeqIO;
+	my $format = Bio::SeqIO->_guess_format($filename);
+
+	if (length $format){
+	    return $format;
+	}else{
+	    warn("Unknown file format. Interpreting $filename as GenBank...\n");
+	    return 'genbank';
+	}
+    }
+}
+
+sub output {
+    my $gb = shift;
+    my $file = shift;
+    my $option = shift;
+
+    $option = _interpret_format($file) unless(length $option);
+
+    if (lc($option) eq 'genbank'){
+	$gb->make_gb($file);
+    }elsif(length $option){
+	my $in;
+
+	if(length $gb->{BIOPERL}){
+	    $in = $gb->{BIOPERL};
+	}else{
+	    my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
+	    $gb->make_gb($outfile);
+	    $in = read_sequence($outfile);
+	}
+
+	write_sequence(">$file", $option, $in);
+    }else{
+	&msg_error("G::output - Unknown format to output.");
+    }
+}
+
+sub bioperl {
+    require Bio::SeqIO;
+    my $this = shift;
+    my ($fh, $outfile) = tempfile(undef, SUFFIX=>'.gbk');
+    $this->output($outfile);
+    
+    my $in = Bio::SeqIO->new(-format => "genbank", -file =>$outfile);
+    my $bp = $in->next_seq();
+
+    return $bp;
+}
+
+
 
 1;
 
