@@ -527,6 +527,7 @@ sub gcsi{
 
  Options:
    -window      window size to observe (default: 10000)
+   -slide       window slide size (default: same as window size)
    -cumulative  1 to calculate cumulative skew (default: 0)
    -at          1 when observing AT skew instead of GC skew (default: 0)
    -purine      1 when observing purine (AG/TC) skew (default: 0)
@@ -541,6 +542,7 @@ sub gcsi{
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
  History:
+   20090601-01 added -slide option
    20070707-01 added -cumulative option
    20060711-01 added purine and keto skew
    20010905-01 update with options
@@ -552,44 +554,45 @@ sub gcsi{
 
 sub gcskew {
     &opt_default(window=>10000, at=>0, purine=>0, keto=>0, output=>"show", 
-		 filename=>"gcskew.png", cumulative=>0);
-    my @args = opt_get(@_);
-    my $gb = opt_as_gb(shift @args);
-    my $ref = \$gb->{SEQ};
-    my $window = opt_val("window");
-    my $output = opt_val("output");
+		 filename=>"gcskew.png", cumulative=>0, slide=>undef);
+    my @args       = opt_get(@_);
+    my $gb         = opt_as_gb(shift @args);
+    my $ref        = \$gb->{SEQ};
+    my $len        = length($$ref);
+    my $window     = opt_val("window");
+    my $slide      = opt_val("slide") || $window;
+    die('Error at gcskew(): window size too small') if ($window < 10);
+    my $output     = opt_val("output");
     my $cumulative = '';
-    $cumulative = "Cumulative " if (opt_val("cumulative"));
-    my $filename = opt_val("filename");
-    $filename =~ s/\.png$/\.csv/ if (opt_val("output") eq 'f');
-    my $at = opt_val("at");
-    my $purine = opt_val("purine");
-    my $keto = opt_val("keto");
-    my @gcskew = ();
-    my @location = ();
-    my $j;
-    my $tmp = 0;
+    $cumulative    = "Cumulative " if (opt_val("cumulative"));
+    my $filename   = opt_val("filename");
+    $filename      =~ s/\.png$/\.csv/ if (opt_val("output") eq 'f');
+    my $at         = opt_val("at");
+    my $purine     = opt_val("purine");
+    my $keto       = opt_val("keto");
+    my @gcskew     = ();
+    my @location   = ();
+    my ($tmp, $pos, $j, $i) = (0,0,0,0);
 
-    my $i = 0;
-    while(length($$ref) - ($window * $i) >= $window){
+    while($len - $pos >= $window){
 	my ($g, $c);
 
 	if($at){
-	    $g = substr($$ref, $window * $i, $window) =~ tr/a/a/;
-	    $c = substr($$ref, $window * $i, $window) =~ tr/t/t/;
+	    $g =  substr($$ref, $pos, $window) =~ tr/a/a/;
+	    $c =  substr($$ref, $pos, $window) =~ tr/t/t/;
 	}elsif($purine){
-	    $g = substr($$ref, $window * $i, $window) =~ tr/a/a/;
-	    $g += substr($$ref, $window * $i, $window) =~ tr/g/g/;
-	    $c = substr($$ref, $window * $i, $window) =~ tr/t/t/;
-	    $c += substr($$ref, $window * $i, $window) =~ tr/c/c/;
+	    $g =  substr($$ref, $pos, $window) =~ tr/a/a/;
+	    $g += substr($$ref, $pos, $window) =~ tr/g/g/;
+	    $c =  substr($$ref, $pos, $window) =~ tr/t/t/;
+	    $c += substr($$ref, $pos, $window) =~ tr/c/c/;
 	}elsif($keto){
-	    $g = substr($$ref, $window * $i, $window) =~ tr/t/t/;
-	    $g += substr($$ref, $window * $i, $window) =~ tr/g/g/;
-	    $c = substr($$ref, $window * $i, $window) =~ tr/a/a/;
-	    $c += substr($$ref, $window * $i, $window) =~ tr/c/c/;
+	    $g =  substr($$ref, $pos, $window) =~ tr/t/t/;
+	    $g += substr($$ref, $pos, $window) =~ tr/g/g/;
+	    $c =  substr($$ref, $pos, $window) =~ tr/a/a/;
+	    $c += substr($$ref, $pos, $window) =~ tr/c/c/;
 	}else{
-	    $g = substr($$ref, $window * $i, $window) =~ tr/g/g/;
-	    $c = substr($$ref, $window * $i, $window) =~ tr/c/c/;
+	    $g =  substr($$ref, $pos, $window) =~ tr/g/g/;
+	    $c =  substr($$ref, $pos, $window) =~ tr/c/c/;
 	}
 
 	if(length($cumulative)){
@@ -607,7 +610,8 @@ sub gcskew {
 	    }
 	}
 
-	$location[$i] = $i * $window;
+	$location[$i] = $pos;
+	$pos += $slide;
 	$i ++;
     }
     $i --;
@@ -620,7 +624,7 @@ sub gcskew {
     }elsif($keto){
 	$title = $cumulative . "Keto skew";
     }
-    
+
     if ($output eq 'g' || $output eq 'show'){
 	mkdir ("graph", 0777);
 
