@@ -35,11 +35,26 @@ use SelfLoader;
 
 our @EXPORT = qw(
 	     over_lapping_finder
-	     _over_lapping_printer
 );
 
 __DATA__
 
+#:::::::::::::::::::::::::::::::::
+#       Perldoc
+#:::::::::::::::::::::::::::::::::
+
+
+=head1 NAME
+
+  G::Seq::Overlapping - Analysis methods related to overlapping genes
+
+=head1 DESCRIPTION
+
+    This class is a part of G-language Genome Analysis Environment, 
+    collecting sequence analysis methods related to overlapping genes.
+
+=cut
+
 #::::::::::::::::::::::::::::::
 #        Methods Start
 #::::::::::::::::::::::::::::::
@@ -73,14 +88,7 @@ sub over_lapping_finder{
     my $output=opt_val("output");
     my $filename=opt_val("filename");
     my $num=1;
-    my $tmp;
-    my $start;
-    my $end;
-    my %result;
-    my $q;
-    my $over;
-    my $switch;
-    my $type;
+    my ($tmp, $start, $end, %result, $q, $over, $switch, $type);
 
     foreach($gb->cds()){
 	$over=0;
@@ -182,38 +190,4 @@ sub _over_lapping_printer{
     }
 }
 
-
-sub DESTROY {
-    my $self = shift;
-}
-
 1;
-__END__
-# Below is the stub of documentation for your module. You better edit it!
-
-=head1 NAME
-
-G::Seq::OverLapping - Perl extension for blah blah blah
-
-=head1 SYNOPSIS
-
-  use G::Seq::OverLapping;
-  blah blah blah
-
-=head1 DESCRIPTION
-
-Stub documentation for G::Seq::OverLapping was created by h2xs. It looks like the
-author of the extension was negligent enough to leave the stub
-unedited.
-
-Blah blah blah.
-
-=head1 AUTHOR
-
-A. U. Thor, a.u.thor@a.galaxy.far.far.away
-
-=head1 SEE ALSO
-
-perl(1).
-
-=cut
