@@ -29,6 +29,8 @@ package G::Seq::PatSearch;
 use SubOpt;
 use G::Messenger;
 use G::Tools::Graph;
+use G::Tools::GMap;
+use G::Tools::Statistics;
 use G::Seq::Primitive;
 
 use strict;
@@ -47,6 +49,8 @@ our @EXPORT = qw(
 	     find_iteron
 	     baseParingTest
 	     nucleotide_periodicity
+	     kmer_table
+	     cgr
 );
 
 =head1 NAME
@@ -309,9 +313,10 @@ sub palindrome {
     @position = find_dif($sequence)
     
   Options:
-    -type    ecoli for E.coli dif (default)
-             proteobacteria, bsub, firmicute, actinobacteria, for corresponding dif sequences.
-    
+    -type      ecoli for E.coli dif (default)
+               proteobacteria, bsub, firmicute, actinobacteria, for corresponding dif sequences.
+    -output    stdout to print data (default: stdout)    
+
   References:
     1. Hendrickson H, Lawrence JG (2007) "Mutational bias suggests that replication 
        termination occurs near the dif site, not at Ter sites",  Mol Microbiol. 64(1):42-56.
@@ -364,8 +369,10 @@ sub find_dif {
     @position = find_ter($sequence)
     
   Options:
-    -type    ecoli for E.coli ter (default)
-             bsub, for corresponding ter sequence.
+    -type      ecoli for E.coli ter (default)
+               bsub, for corresponding ter sequence.
+    -output    stdout to print data (default: stdout)
+    
 
   References:
     1. Hendrickson H, Lawrence JG (2007) "Mutational bias suggests that replication 
@@ -802,6 +809,441 @@ sub nucleotide_periodicity {
 
 
 
+
+
+=head2 kmer_table
+
+ Name: kmer_table   -   create an image showing all k-mer abundance within a sequence
+
+  Description:
+    This program creates an image showing the abundance of all k-mers (oligonucleotides
+    of length k) in a given sequence. For example, for tetramers (k=4), resulting image
+    is composed of 4^4 = 256 boxes, each representing an oligomer. Oligomer name and 
+    abundance is written within these boxes, and abundance is also visualized with 
+    the box color, from white (none) to black (highly frequent).
+
+    This k-mer table is alternatively known as the FCGR (frequency matrices extracted
+    from Chaos Game Representation).
+
+    Position of the oligomers can be recursively located as follows:
+    For each letter in an oligomer, a box is subdivided into four quadrants, 
+    where A is upper left, T is lower right, G is upper right, and C is lower left.
+    Therefore, oligomer ATGC is in the 
+      A = upper left quadrant
+      T = lower right within the above quadrant
+      G = upper right within the above quadrant
+      C = lower left within the above quadrant
+
+    With Google Maps representation using -gmap=>1 option, oligomers can be searched
+    incrementally from the top search box.
+
+    More detailed documentation is available at 
+    http://www.g-language.org/wiki/cgr
+ 
+  Usage: 
+    null = kmer_table($sequence);
+    
+  Options:
+    -k         length of oligomer (default: 6)
+    -filename  filename for output image (default: kmer.png)
+    -level     zoom level for Google Maps representation (default: 5)
+    -gmap      set to 1 to use Google Maps API to create zoomable image (default: 0)
+    -output    "g" for image output, "show" for image output and display, 
+               "gmap" for zoomable image using Google Maps API (default: show)    
+
+  References:
+    1. Almeida JS, Carrico JA, Maretzek A, Noble PA, Fletcher M (2001) 
+       "Analysis of genomic sequences by Chaos Game Representation." 
+       Bioinformatics. 17(5):429-437.
+  
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+    
+  History:
+    20090716-01 initial posting
+
+=cut
+
+
+sub kmer_table {
+    require GD;
+
+    opt_default('output'=>'show', 'k'=>6, 'filename'=>'kmer.png', 'gmap'=>0, 'level'=>5);
+    my @argv      = opt_get(@_);
+    my $gb        = opt_as_gb(shift @argv);
+    my $filename  = opt_val('filename');
+    my $output    = opt_val('output');
+    my $oligo_num = opt_val('k');
+    my $gmap      = opt_val('gmap');
+    my $level     = opt_val('level');
+    my $apikey    = opt_val('apikey');
+    $output       = 'gmap' if($gmap);
+    $gmap         = 1 if($output eq 'gmap');
+
+    my $font;
+    $font->{a} = [
+		    [2,1],
+	      [1,2],      [3,2],
+	      [1,3],[2,3],[3,3],
+	      [1,4],      [3,4]
+	      ];
+
+    $font->{t} = [
+	      [1,1],[2,1],[3,1],
+	            [2,2],
+	            [2,3],
+	            [2,4],
+	      ];
+
+    $font->{g} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],
+	      [1,3],      [3,3],
+	      [1,4],[2,4],[3,4]
+	      ];
+
+    $font->{c} = [
+	            [2,1],[3,1],
+	      [1,2],
+	      [1,3],
+	            [2,4],[3,4]
+	      ];
+
+    $font->{k} = [
+	      [1,1],      [3,1],
+	      [1,2],[2,2],
+	      [1,3],      [3,3],
+	      [1,4],      [3,4]
+	      ];
+
+    $font->{m} = [
+	      [1,1],                  [5,1],
+	      [1,2],[2,2],      [4,2],[5,2],
+	      [1,3],      [3,3],      [5,3],
+	      [1,4],                  [5,4]
+	      ];
+
+    $font->{null} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],[2,2],[3,2],
+	      [1,3],[2,3],[3,3],
+	      [1,4],[2,4],[3,4]
+	      ];
+
+
+    $font->{0} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],      [3,2],
+	      [1,3],      [3,3],
+	      [1,4],      [3,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{1} = [
+	      [1,1],[2,1],
+	            [2,2],
+	            [2,3],
+	            [2,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{2} = [
+	      [1,1],[2,1],[3,1],
+	                  [3,2],
+	      [1,3],[2,3],[3,3],
+	      [1,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{3} = [
+	      [1,1],[2,1],[3,1],
+	                  [3,2],
+	      [1,3],[2,3],[3,3],
+	                  [3,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{4} = [
+	      [1,1],      [3,1],
+	      [1,2],      [3,2],
+	      [1,3],[2,3],[3,3],
+	                  [3,4],
+	                  [3,5]
+	      ];
+
+    $font->{5} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],
+	      [1,3],[2,3],[3,3],
+	                  [3,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{6} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],
+	      [1,3],[2,3],[3,3],
+	      [1,4],      [3,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{7} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],      [3,2],
+	                  [3,3],
+	                  [3,4],
+	                  [3,5]
+	      ];
+
+    $font->{8} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],      [3,2],
+	      [1,3],[2,3],[3,3],
+	      [1,4],      [3,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    $font->{9} = [
+	      [1,1],[2,1],[3,1],
+	      [1,2],      [3,2],
+	      [1,3],[2,3],[3,3],
+	                  [3,4],
+	      [1,5],[2,5],[3,5]
+	      ];
+
+    my $separate = $oligo_num * 4;
+    $separate = 32 if($separate < 32);
+
+    my %colorTable = oligomer_counter($gb,-length=>$oligo_num);
+    for (my $i=0;$i<4**$oligo_num;$i++) {
+	my $num = $i;
+	my @seq = ();
+	for (1 .. $oligo_num) {
+	    my $tmp = int($num/(4**($oligo_num - $_)));
+	    $num = $num%(4**($oligo_num - $_));
+	    if ($tmp == 0) {
+		$seq[$_] = 'a';
+	    } elsif ($tmp == 1) {
+		$seq[$_] = 't';
+	    } elsif ($tmp == 2) {
+		$seq[$_] = 'g';
+	    } elsif ($tmp == 3) {
+		$seq[$_] = 'c';
+	    } else {
+		$seq[$_] = $tmp;
+	    }
+	}
+	my $seq = join('',@seq);
+	if ($colorTable{$seq} eq '') {
+	    $colorTable{$seq} = 0;
+	}
+    }
+
+    my $max = max(values %colorTable);
+    my $wide = 2 ** $oligo_num * $separate;
+    my $img = GD::Image->new($wide, $wide);
+
+    my @color;
+    for (0..25) {
+	my $val = $_ * 10 + 5;
+	$color[$_] = $img->colorAllocate($val, $val, $val);
+    }
+    
+    $img->fill($wide - 1, $wide - 1, $color[25]);
+
+    for my $seq (keys(%colorTable)) {
+	
+	my $up_down = $seq;
+	$up_down =~ tr/acgt/0011/;
+	
+	my $r_l = $seq;
+	$r_l =~ tr/agct/0011/;
+	
+	my $x = oct("0b" . $up_down) * $separate;
+	my $y = oct("0b" . $r_l) * $separate;
+	
+	my $contrast = int((255 - 255 * ($colorTable{$seq} / $max))/10);
+	$contrast = 25 if ($colorTable{$seq} == 0);
+	
+	my $color = $color[$contrast];
+	$img->rectangle($x, $y, $x + $separate-1, $y + $separate-1, $color);
+	$img->fill($x+1, $y+1, $color);
+	
+	my $base_color;
+	if ($contrast < 14){# $max/2 < $colorTable{$seq}) {
+	    $base_color = $color[20];
+	} else {
+	    $base_color = $color[7];
+	}
+	
+	my $i = 0;
+	for my $tmp (split(//, $seq)){
+	    $img->setPixel($x + $font->{$tmp}[$_][0] + $i * 4, $y + 3 + $font->{$tmp}[$_][1], $base_color) for(0..scalar(@{$font->{$tmp}})-1);
+	    $i ++;
+	}
+	
+	my $num = $colorTable{$seq};
+	if    ($oligo_num <= 6 && length($num) - 8 > 5 || $oligo_num > 4 && length($num) - $oligo_num > 5){
+	    substr($num, -9) = 'g';
+	}elsif($oligo_num <= 6 && length($num) - 8 > 2 || $oligo_num > 4 && length($num) - $oligo_num > 2){
+	    substr($num, -6) = 'm';
+	}elsif($oligo_num <= 6 && length($num) - 8 > 0 || $oligo_num > 4 && length($num) - $oligo_num > 0){
+	    substr($num, -3) = 'k';
+	}
+	
+	$i = 0;
+	for my $tmp (split(//, $num)){
+	    $img->setPixel($x + $font->{$tmp}[$_][0] + $i * 4, $y + 10 + $font->{$tmp}[$_][1], $base_color) for(0..scalar(@{$font->{$tmp}})-1);
+	    $i ++;
+	}
+    }
+    
+    binmode STDOUT;
+
+    open  FILE,">graph/$filename";
+    print FILE $img->png;
+    close FILE;
+
+    msg_gimv("graph/$filename") if ($output eq 'show');
+
+    if($gmap){
+        generateGMap("graph/$filename", -cgr=>1, -level=>$level, -apikey=>$apikey);
+    }
+}
+
+
+
+
+
+=head2 cgr
+
+ Name: cgr   -   create a Chaos Game Representation of a given sequence
+
+  Description:
+    This program creates a Chaos Game Representation (CGR) image of a given sequence.
+
+    CGR is generated by the following procedure:
+    1. Start from position (0,0) or the origin of two dimensional coordinate.
+       Four nucleotides are located at the four corners: 
+           A: (-1, 1)  upper left
+           T: (1, -1)  lower right
+           G: (1, 1)   upper right
+           C: (-1, -1) lower left
+    2. For each nucleotide, move and mark the new location which is halfway between 
+       the current location and the nucleotide. 
+       For example, if the first letter is A, position is moved from (0,0) to 
+       midpoint between (-1, 1) and (0,0), which is (-0.5, 0.5).
+    3. Repeat this procedure for all nucleotides.
+
+    CGR is a generalized scale-independent Markov probability table for the sequence, 
+    and oligomer tables (see document for "kmer_table" function) can be deduced from CGR image.
+
+    With Google Maps representation using -gmap=>1 option, oligomers can be searched
+    incrementally from the top search box.
+
+    More detailed documentation is available at 
+    http://www.g-language.org/wiki/cgr
+ 
+  Usage: 
+    null = kmer_table($sequence);
+    
+  Options:
+    -width     width of image (default: 1024)
+    -filename  filename for output image (default: cgr.png)
+    -level     zoom level for Google Maps representation (default: 5)
+    -gmap      set to 1 to use Google Maps API to create zoomable image (default: 0)
+    -output    "g" for image output, "show" for image output and display, 
+               "gmap" for zoomable image using Google Maps API (default: show)    
+
+  References:
+    1. Almeida JS, Carrico JA, Maretzek A, Noble PA, Fletcher M (2001) 
+       "Analysis of genomic sequences by Chaos Game Representation." 
+       Bioinformatics. 17(5):429-437.
+  
+  Author: 
+    Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
+    
+  History:
+    20090716-01 initial posting
+
+=cut
+
+
+sub cgr {
+    require GD;
+
+    opt_default('output'=>'show', 'width'=>1024, 'filename'=>'cgr.png', 'gmap'=>0, 'level'=>5);
+    my @argv = opt_get(@_);
+    my $gb = opt_as_gb(shift @argv);
+
+    my $output   = opt_val('output');
+    my $filename = opt_val('filename');
+    my $wide     = opt_val('width');
+    my $gmap     = opt_val('gmap');
+    my $level    = opt_val('level');
+    my $apikey   = opt_val('apikey');
+    $output      = 'gmap' if($gmap);
+    $gmap        = 1 if($output eq 'gmap');
+    $wide        = 2 ** ($level - 1) * 256 if($gmap);
+    my $adjust   = int(sqrt($wide)) - 8;
+    $adjust      = 1 if($adjust < 1);
+
+    my @pixel;
+    for my $x (0..$wide-1) {
+	for my $y (0..$wide-1) {
+	    $pixel[$x][$y] = 0;
+	}
+    }
+    
+    my $img = GD::Image->new($wide,$wide);
+
+    my @color;
+    for (0..10){
+	my $val = $_ * 25 + 5;
+	$color[$_] = $img->colorAllocate($val, $val, $val);
+    }
+
+    my @pos = (int(($wide-1)/2), int(($wide-1)/2));
+    my $max = 0;
+    for my $base (reverse(split(//, lc($gb->{SEQ})))){
+	if ($base eq 'a') {
+	    $pos[0] = int($pos[0]/2);
+	    $pos[1] = int($pos[1]/2);
+	} elsif ($base eq 't') {
+	    $pos[0] = int(($pos[0]+$wide-1)/2);
+	    $pos[1] = int(($pos[1]+$wide-1)/2);
+	} elsif ($base eq 'g') {
+	    $pos[0] = int(($pos[0]+$wide-1)/2);
+	    $pos[1] = int($pos[1]/2);
+	} elsif ($base eq 'c') {
+	    $pos[0] = int($pos[0]/2);
+	    $pos[1] = int(($pos[1]+$wide-1)/2);
+	}
+	$pixel[$pos[0]][$pos[1]]++;
+	$max = $pixel[$pos[0]][$pos[1]] if ($max < $pixel[$pos[0]][$pos[1]]);
+    }
+    
+    for my $x (0..$wide-1) {
+	for my $y (0..$wide-1) {
+	    my $colorcode = int(10-$pixel[$x][$y]/$max*$adjust*10);
+	    $colorcode = 0 if ($colorcode < 0);
+	    $img->setPixel($x, $y, $color[$colorcode]);
+	}
+    }
+    
+    binmode STDOUT;
+    
+    open  FILE,">graph/$filename";
+    print FILE $img->png;
+    close FILE;
+
+    msg_gimv("graph/$filename") if ($output eq 'show');
+
+    if($gmap){
+        generateGMap("graph/$filename", -cgr=>1, -level=>$level, -apikey=>$apikey);
+    }
+}
+
 1;
 
 
