@@ -174,9 +174,7 @@ sub splitprintseq {
     This method outputs the given sequence as a FASTA file.
     
   Usage : 
-    to_fasta($seq, -name=>"My sequence"); # output the sequence to out.fasta
-      or
-    $fasta_string = to_fasta($gb, -output=>"return");
+    $fasta_string = to_fasta($seq, -name=>"My sequence"); # output the sequence to out.fasta
 
  Options:
     -name        string for FASTA header (default: sequence)
@@ -184,13 +182,14 @@ sub splitprintseq {
                  $gb->{LOCUS}->{id} is used as default
     -length      number of characters per line (default: 60)
     -filename    output filename (default: "out.fasta")
-    -output      "f" to output to file, and return the filename.
-                 "return" to return the fasta as a string
+    -output      "f" to output to file, and return the fasta as a string.
+                 "return" to just return the fasta as a string
 
   Author: 
     Kazuharu Arakawa (gaou@sfc.keio.ac.jp)
 
   History:
+    20090312-01 changed the deafult behavior to return fasta as a string as well as outputting to a file
     20070612-01 added -output option
     20050116-01 initial posting
 
@@ -213,7 +212,7 @@ sub to_fasta{
 	open(OUT, ">$filename") || die($!);
 	printf OUT ">%s\n%s", $name, splitprintseq($gb->{SEQ}, $length);
 	close(OUT);
-	return $filename;
+	return sprintf ">%s\n%s", $name, splitprintseq($gb->{SEQ}, $length);
     }else{
 	return sprintf ">%s\n%s", $name, splitprintseq($gb->{SEQ}, $length);
     }
