@@ -125,6 +125,8 @@ sub togoWS{
 		$opt{db} = 'glycan';
 	    }elsif($args[0] =~ /^R\d+$/){
 		$opt{db} = 'reaction';
+	    }elsif($args[0] =~ /^NC_\d+$/){
+		$opt{db} = 'genbank';
 	    }elsif($args[0] =~ /^BIOMD\d+$/){
 		$opt{db} = 'biomodels';
 	    }elsif($args[0] =~ /^MODEL\d+$/){
