@@ -36,13 +36,14 @@ use strict;
 use base qw(Exporter);
 
 our @EXPORT = qw(
+		 opt_inputType
 		 opt_default
 		 opt_val
 		 opt_get
 		 opt_as_gb
 );
 
-our $VERSION = '0.40';
+our $VERSION = '0.60';
 
 #::::::::::::::::::::::::::::::
 #          Variables
@@ -60,6 +61,10 @@ sub set_opt_list{
     $opt_list = shift;
 }
 
+sub opt_inputType{
+
+}
+
 sub opt_as_gb {
     return if ($opt_list);
 
@@ -122,9 +127,6 @@ sub opt_get{
     my $i = 0;
     my @bparray = ();
 
-    mkdir ("data", 0777);
-    mkdir ("graph", 0777);
-
     while(defined $args[$i]){
 	if (substr($args[$i], 0, 1) eq '-' && substr($args[$i], 1, 1) !~ /[0-9]/){
 	    if(!defined($args[$i + 1]) || substr($args[$i + 1], 0, 1) eq '-' && substr($args[$i + 1], 1, 1) !~ /[0-9]/){
@@ -145,6 +147,9 @@ sub opt_get{
 	}
     }
 
+    mkdir ("data",  0777) if($data{'output'} eq 'f');;
+    mkdir ("graph", 0777) if($data{'output'} =~ /g/ || $data{'output'} eq 'show');
+
     return @new_args;
 }
 
